@@ -226,10 +226,9 @@ PyDoc_STRVAR(Connection_close__doc__,
 "close()\n\n"
 "Close the connection.");
 static PyObject *
-Connection_close (Connection *self, PyObject *args)
+Connection_close (Connection *self, PyObject *args UNUSED)
 {
     TRACE(self);
-    if (!PyArg_ParseTuple(args, ":close")) return NULL;
     /* Because the user explicitly asked to close the connection, we'll even
     let them close shared connections. */
     if (self->conn) {
@@ -244,13 +243,12 @@ PyDoc_STRVAR(Connection_get_is_connected__doc__,
 "get_is_connected() -> bool\n\n"
 "Return true if this Connection is connected.\n");
 static PyObject *
-Connection_get_is_connected (Connection *self, PyObject *args)
+Connection_get_is_connected (Connection *self, PyObject *args UNUSED)
 {
     dbus_bool_t ret;
 
     TRACE(self);
     DBUS_PY_RAISE_VIA_NULL_IF_FAIL(self->conn);
-    if (!PyArg_ParseTuple(args, ":get_is_connected")) return NULL;
     Py_BEGIN_ALLOW_THREADS
     ret = dbus_connection_get_is_connected(self->conn);
     Py_END_ALLOW_THREADS
@@ -261,13 +259,12 @@ PyDoc_STRVAR(Connection_get_is_authenticated__doc__,
 "get_is_authenticated() -> bool\n\n"
 "Return true if this Connection was ever authenticated.\n");
 static PyObject *
-Connection_get_is_authenticated (Connection *self, PyObject *args)
+Connection_get_is_authenticated (Connection *self, PyObject *args UNUSED)
 {
     dbus_bool_t ret;
 
     TRACE(self);
     DBUS_PY_RAISE_VIA_NULL_IF_FAIL(self->conn);
-    if (!PyArg_ParseTuple(args, ":get_is_authenticated")) return NULL;
     Py_BEGIN_ALLOW_THREADS
     ret = dbus_connection_get_is_authenticated(self->conn);
     Py_END_ALLOW_THREADS
@@ -406,6 +403,13 @@ Connection_send_message_with_reply(Connection *self, PyObject *args, PyObject *k
         return PyErr_NoMemory();
     }
 
+    if (!pending) {
+        /* connection is disconnected (doesn't return FALSE!) */
+        PyErr_SetString (DBusPyException, "Connection is disconnected - "
+                                          "unable to make method call");
+        return NULL;
+    }
+
     return DBusPyPendingCall_ConsumeDBusPendingCall(pending, callable);
 }
 
@@ -682,7 +686,7 @@ Connection__register_object_path(Connection *self, PyObject *args,
     }
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
                                      "OO|Oi:_register_object_path",
-                                     argnames, 
+                                     argnames,
                                      &path,
                                      &on_message, &on_unregister,
                                      &fallback)) return NULL;
@@ -854,7 +858,7 @@ Connection__unregister_object_path(Connection *self, PyObject *args,
         Py_DECREF(path);
         return NULL;
     }
-    
+
     /* BEGIN PARANOIA
     This is something of a critical section - the dict of object-paths
     and libdbus' internal structures are out of sync for a bit. We have
@@ -910,6 +914,27 @@ Connection__unregister_object_path(Connection *self, PyObject *args,
 
 /* dbus_connection_get_outgoing_size - almost certainly unneeded */
 
+PyDoc_STRVAR(new_for_bus__doc__,
+"Connection._new_for_bus([address: str or int]) -> Connection\n"
+"\n"
+"If the address is an int it must be one of the constants BUS_SESSION,\n"
+"BUS_SYSTEM, BUS_STARTER; if a string, it must be a D-Bus address.\n"
+"The default is BUS_SESSION.\n"
+);
+
+PyDoc_STRVAR(get_unique_name__doc__,
+"get_unique_name() -> str\n\n"
+"Return this application's unique name on this bus.\n"
+"\n"
+":Raises DBusException: if the connection has no unique name yet\n"
+"   (for Bus objects this can't happen, for peer-to-peer connections\n"
+"   this means you haven't called `set_unique_name`)\n");
+
+PyDoc_STRVAR(set_unique_name__doc__,
+"set_unique_name(str)\n\n"
+"Set this application's unique name on this bus. Raise ValueError if it has\n"
+"already been set.\n");
+
 struct PyMethodDef DBusPyConnection_tp_methods[] = {
 #define ENTRY(name, flags) {#name, (PyCFunction)Connection_##name, flags, Connection_##name##__doc__}
     ENTRY(_require_main_loop, METH_NOARGS),
@@ -928,6 +953,15 @@ struct PyMethodDef DBusPyConnection_tp_methods[] = {
     ENTRY(send_message_with_reply, METH_VARARGS|METH_KEYWORDS),
     ENTRY(send_message_with_reply_and_block, METH_VARARGS),
     ENTRY(_unregister_object_path, METH_VARARGS|METH_KEYWORDS),
+    {"_new_for_bus", (PyCFunction)DBusPyConnection_NewForBus,
+        METH_CLASS|METH_VARARGS|METH_KEYWORDS,
+        new_for_bus__doc__},
+    {"get_unique_name", (PyCFunction)DBusPyConnection_GetUniqueName,
+        METH_NOARGS,
+        get_unique_name__doc__},
+    {"set_unique_name", (PyCFunction)DBusPyConnection_SetUniqueName,
+        METH_VARARGS,
+        set_unique_name__doc__},
     {NULL},
 #undef ENTRY
 };
