@@ -36,31 +36,9 @@ __docformat__ = 'restructuredtext'
 
 _logger = logging.getLogger('dbus.proxies')
 
-
-BUS_DAEMON_NAME = 'org.freedesktop.DBus'
-BUS_DAEMON_PATH = '/org/freedesktop/DBus'
-BUS_DAEMON_IFACE = BUS_DAEMON_NAME
-
-
-class _ReplyHandler(object):
-    __slots__ = ('_on_error', '_on_reply', '_get_args_options')
-    def __init__(self, on_reply, on_error, **get_args_options):
-        self._on_error = on_error
-        self._on_reply = on_reply
-        self._get_args_options = get_args_options
-
-    def __call__(self, message):
-        if isinstance(message, _dbus_bindings.MethodReturnMessage):
-            self._on_reply(*message.get_args_list(**self._get_args_options))
-        elif isinstance(message, _dbus_bindings.ErrorMessage):
-            args = message.get_args_list()
-            if len(args) > 0:
-                self._on_error(DBusException(args[0]))
-            else:
-                self._on_error(DBusException())
-        else:
-            self._on_error(DBusException('Unexpected reply message type: %s'
-                                        % message))
+from _dbus_bindings import LOCAL_PATH, \
+                           BUS_DAEMON_NAME, BUS_DAEMON_PATH, BUS_DAEMON_IFACE,\
+                           INTROSPECTABLE_IFACE
 
 
 class _DeferredMethod:
@@ -84,6 +62,9 @@ class _DeferredMethod:
             self._block()
             return self._proxy_method(*args, **keywords)
 
+    def call_async(self, *args, **keywords):
+        self._append(self._proxy_method, args, keywords)
+
 
 class _ProxyMethod:
     """A proxy method.
@@ -92,91 +73,91 @@ class _ProxyMethod:
     method produce messages that travel over the Bus and are routed
     to a specific named Service.
     """
-    def __init__(self, proxy, connection, named_service, object_path, method_name, iface):
+    def __init__(self, proxy, connection, bus_name, object_path, method_name,
+                 iface):
+        if object_path == LOCAL_PATH:
+            raise DBusException('Methods may not be called on the reserved '
+                                'path %s' % LOCAL_PATH)
+
+        # trust that the proxy, and the properties it had, are OK
         self._proxy          = proxy
         self._connection     = connection
-        self._named_service  = named_service
+        self._named_service  = bus_name
         self._object_path    = object_path
+        # fail early if the method name is bad
+        _dbus_bindings.validate_member_name(method_name)
         # the test suite relies on the existence of this property
         self._method_name    = method_name
+        # fail early if the interface name is bad
+        if iface is not None:
+            _dbus_bindings.validate_interface_name(iface)
         self._dbus_interface = iface
 
     def __call__(self, *args, **keywords):
-        timeout = -1
-        if keywords.has_key('timeout'):
-            timeout = keywords['timeout']
+        reply_handler = keywords.pop('reply_handler', None)
+        error_handler = keywords.pop('error_handler', None)
+        ignore_reply = keywords.pop('ignore_reply', False)
 
-        reply_handler = None
-        if keywords.has_key('reply_handler'):
-            reply_handler = keywords['reply_handler']
-
-        error_handler = None
-        if keywords.has_key('error_handler'):
-            error_handler = keywords['error_handler']
-
-        ignore_reply = False
-        if keywords.has_key('ignore_reply'):
-            ignore_reply = keywords['ignore_reply']
-
-        get_args_options = {}
-        if keywords.has_key('utf8_strings'):
-            get_args_options['utf8_strings'] = keywords['utf8_strings']
-        if keywords.has_key('byte_arrays'):
-            get_args_options['byte_arrays'] = keywords['byte_arrays']
-
-        if not(reply_handler and error_handler):
-            if reply_handler:
+        if reply_handler is not None or error_handler is not None:
+            if reply_handler is None:
                 raise MissingErrorHandlerException()
-            elif error_handler:
+            elif error_handler is None:
                 raise MissingReplyHandlerException()
+            elif ignore_reply:
+                raise TypeError('ignore_reply and reply_handler cannot be '
+                                'used together')
 
-        dbus_interface = self._dbus_interface
-        if keywords.has_key('dbus_interface'):
-            dbus_interface = keywords['dbus_interface']
+        dbus_interface = keywords.pop('dbus_interface', self._dbus_interface)
 
-        tmp_iface = ''
-        if dbus_interface:
-            tmp_iface = dbus_interface + '.'
-
-        key = tmp_iface + self._method_name
-
-        introspect_sig = None
-        if self._proxy._introspect_method_map.has_key (key):
-            introspect_sig = self._proxy._introspect_method_map[key]
-
-        message = _dbus_bindings.MethodCallMessage(destination=None,
-                                                   path=self._object_path,
-                                                   interface=dbus_interface,
-                                                   method=self._method_name)
-        message.set_destination(self._named_service)
-        
-        # Add the arguments to the function
-        try:
-            message.append(signature=introspect_sig, *args)
-        except Exception, e:
-            _logger.error('Unable to set arguments %r according to '
-                          'introspected signature %r: %s: %s',
-                          args, introspect_sig, e.__class__, e)
-            raise
-
-        if ignore_reply:
-            self._connection.send_message(message)
-            return None
-        elif reply_handler:
-            self._connection.send_message_with_reply(message, _ReplyHandler(reply_handler, error_handler, **get_args_options), timeout/1000.0, require_main_loop=1)
-            return None
+        if dbus_interface is None:
+            key = self._method_name
         else:
-            reply_message = self._connection.send_message_with_reply_and_block(message, timeout)
-            args_list = reply_message.get_args_list(**get_args_options)
-            if len(args_list) == 0:
-                return None
-            elif len(args_list) == 1:
-                return args_list[0]
-            else:
-                return tuple(args_list)
+            key = dbus_interface + '.' + self._method_name
+        introspect_sig = self._proxy._introspect_method_map.get(key, None)
+
+        if ignore_reply or reply_handler is not None:
+            self._connection.call_async(self._named_service,
+                                        self._object_path,
+                                        dbus_interface,
+                                        self._method_name,
+                                        introspect_sig,
+                                        args,
+                                        reply_handler,
+                                        error_handler,
+                                        **keywords)
+        else:
+            return self._connection.call_blocking(self._named_service,
+                                                  self._object_path,
+                                                  dbus_interface,
+                                                  self._method_name,
+                                                  introspect_sig,
+                                                  args,
+                                                  **keywords)
+
+    def call_async(self, *args, **keywords):
+        reply_handler = keywords.pop('reply_handler', None)
+        error_handler = keywords.pop('error_handler', None)
+
+        dbus_interface = keywords.pop('dbus_interface', self._dbus_interface)
+
+        if dbus_interface:
+            key = dbus_interface + '.' + self._method_name
+        else:
+            key = self._method_name
+        introspect_sig = self._proxy._introspect_method_map.get(key, None)
+
+        self._connection.call_async(self._named_service,
+                                    self._object_path,
+                                    dbus_interface,
+                                    self._method_name,
+                                    introspect_sig,
+                                    args,
+                                    reply_handler,
+                                    error_handler,
+                                    **keywords)
 
 
-class ProxyObject:
+class ProxyObject(object):
     """A proxy to the remote Object.
 
     A ProxyObject is provided by the Bus. ProxyObjects
@@ -189,51 +170,74 @@ class ProxyObject:
     INTROSPECT_STATE_INTROSPECT_IN_PROGRESS = 1
     INTROSPECT_STATE_INTROSPECT_DONE = 2
 
-    def __init__(self, bus, named_service, object_path, introspect=True,
-                 follow_name_owner_changes=False):
+    def __init__(self, conn=None, bus_name=None, object_path=None,
+                 introspect=True, follow_name_owner_changes=False, **kwargs):
         """Initialize the proxy object.
 
         :Parameters:
-            `bus` : `dbus.Bus`
-                The bus on which to find this object
-            `named_service` : str
-                A bus name for the endpoint owning the object (need not
-                actually be a service name)
+            `conn` : `dbus.connection.Connection`
+                The bus or connection on which to find this object.
+                The keyword argument `bus` is a deprecated alias for this.
+            `bus_name` : str
+                A bus name for the application owning the object, to be used
+                as the destination for method calls and the sender for
+                signal matches. The keyword argument `named_service` is a
+                deprecated alias for this.
             `object_path` : str
-                The object path at which the endpoint exports the object
+                The object path at which the application exports the object
             `introspect` : bool
                 If true (default), attempt to introspect the remote
                 object to find out supported methods and their signatures
             `follow_name_owner_changes` : bool
-                If true (default is false) and the `named_service` is a
+                If true (default is false) and the `bus_name` is a
                 well-known name, follow ownership changes for that name
         """
-        if follow_name_owner_changes:
-            bus._require_main_loop()   # we don't get the signals otherwise
+        bus = kwargs.pop('bus', None)
+        if bus is not None:
+            if conn is not None:
+                raise TypeError('conn and bus cannot both be specified')
+            conn = bus
+            from warnings import warn
+            warn('Passing the bus parameter to ProxyObject by name is '
+                 'deprecated: please use positional parameters',
+                 DeprecationWarning, stacklevel=2)
+        named_service = kwargs.pop('named_service', None)
+        if named_service is not None:
+            if bus_name is not None:
+                raise TypeError('bus_name and named_service cannot both be '
+                                'specified')
+            bus_name = named_service
+            from warnings import warn
+            warn('Passing the named_service parameter to ProxyObject by name '
+                 'is deprecated: please use positional parameters',
+                 DeprecationWarning, stacklevel=2)
+        if kwargs:
+            raise TypeError('ProxyObject.__init__ does not take these '
+                            'keyword arguments: %s'
+                            % ', '.join(kwargs.iterkeys()))
 
-        self._bus           = bus
-        self._named_service = named_service
+        if follow_name_owner_changes:
+            # we don't get the signals unless the Bus has a main loop
+            # XXX: using Bus internals
+            conn._require_main_loop()
+
+        self._bus = conn
+
+        if bus_name is not None:
+            _dbus_bindings.validate_bus_name(bus_name)
+        # the attribute is still called _named_service for the moment,
+        # for the benefit of telepathy-python
+        self._named_service = self._requested_bus_name = bus_name
+
+        _dbus_bindings.validate_object_path(object_path)
         self.__dbus_object_path__ = object_path
 
-        if (named_service[:1] != ':' and named_service != BUS_DAEMON_NAME
-            and not follow_name_owner_changes):
-            bus_object = bus.get_object(BUS_DAEMON_NAME, BUS_DAEMON_PATH)
-            try:
-                self._named_service = bus_object.GetNameOwner(named_service,
-                        dbus_interface=BUS_DAEMON_IFACE)
-            except DBusException, e:
-                # FIXME: detect whether it's NameHasNoOwner, but properly
-                #if not str(e).startswith('org.freedesktop.DBus.Error.NameHasNoOwner:'):
-                #    raise
-                # it might not exist: try to start it
-                bus_object.StartServiceByName(named_service,
-                                              _dbus_bindings.UInt32(0))
-                self._named_service = bus_object.GetNameOwner(named_service,
-                        dbus_interface=BUS_DAEMON_IFACE)
+        if not follow_name_owner_changes:
+            self._named_service = conn.activate_name_owner(bus_name)
 
         #PendingCall object for Introspect call
         self._pending_introspect = None
-        #queue of async calls waiting on the Introspect to return 
+        #queue of async calls waiting on the Introspect to return
         self._pending_introspect_queue = []
         #dictionary mapping method names to their input signatures
         self._introspect_method_map = {}
@@ -242,13 +246,49 @@ class ProxyObject:
         # and calls the callback which re-takes the lock
         self._introspect_lock = RLock()
 
-        if not introspect:
+        if not introspect or self.__dbus_object_path__ == LOCAL_PATH:
             self._introspect_state = self.INTROSPECT_STATE_DONT_INTROSPECT
         else:
             self._introspect_state = self.INTROSPECT_STATE_INTROSPECT_IN_PROGRESS
-            
+
             self._pending_introspect = self._Introspect()
 
+    bus_name = property(lambda self: self._named_service, None, None,
+            """The bus name to which this proxy is bound. (Read-only,
+            may change.)
+
+            If the proxy was instantiated using a unique name, this property
+            is that unique name.
+
+            If the proxy was instantiated with a well-known name and with
+            `follow_name_owner_changes` set false (the default), this
+            property is the unique name of the connection that owned that
+            well-known name when the proxy was instantiated, which might
+            not actually own the requested well-known name any more.
+
+            If the proxy was instantiated with a well-known name and with
+            `follow_name_owner_changes` set true, this property is that
+            well-known name.
+            """)
+
+    requested_bus_name = property(lambda self: self._requested_bus_name,
+            None, None,
+            """The bus name which was requested when this proxy was
+            instantiated.
+            """)
+
+    object_path = property(lambda self: self.__dbus_object_path__,
+            None, None,
+            """The object-path of this proxy.""")
+
+    # XXX: We don't currently support this because it's the signal receiver
+    # that's responsible for tracking name owner changes, but it
+    # seems a natural thing to add in future.
+    #unique_bus_name = property(lambda self: something, None, None,
+    #        """The unique name of the connection to which this proxy is
+    #        currently bound. (Read-only, may change.)
+    #        """)
+
     def connect_to_signal(self, signal_name, handler_function, dbus_interface=None, **keywords):
         """Arrange for the given function to be called when the given signal
         is received.
@@ -312,17 +352,19 @@ class ProxyObject:
         self._bus.add_signal_receiver(handler_function,
                                       signal_name=signal_name,
                                       dbus_interface=dbus_interface,
-                                      named_service=self._named_service,
+                                      bus_name=self._named_service,
                                       path=self.__dbus_object_path__,
                                       **keywords)
 
     def _Introspect(self):
-        message = _dbus_bindings.MethodCallMessage(None, self.__dbus_object_path__, 'org.freedesktop.DBus.Introspectable', 'Introspect')
-        message.set_destination(self._named_service)
-        
-        result = self._bus.get_connection().send_message_with_reply(message, _ReplyHandler(self._introspect_reply_handler, self._introspect_error_handler, utf8_strings=True), -1)
-        return result
-    
+        return self._bus.call_async(self._named_service,
+                                    self.__dbus_object_path__,
+                                    INTROSPECTABLE_IFACE, 'Introspect', '', (),
+                                    self._introspect_reply_handler,
+                                    self._introspect_error_handler,
+                                    utf8_strings=True,
+                                    require_main_loop=False)
+
     def _introspect_execute_queue(self):
         # FIXME: potential to flood the bus
         # We should make sure mainloops all have idle handlers
@@ -377,13 +419,11 @@ class ProxyObject:
         finally:
             self._introspect_lock.release()
 
-    def __getattr__(self, member, dbus_interface=None):
-        if member == '__call__':
-            return object.__call__
-        elif member.startswith('__') and member.endswith('__'):
+    def __getattr__(self, member):
+        if member.startswith('__') and member.endswith('__'):
             raise AttributeError(member)
         else:
-            return self.get_dbus_method(member, dbus_interface)
+            return self.get_dbus_method(member)
 
     def get_dbus_method(self, member, dbus_interface=None):
         """Return a proxy method representing the given D-Bus method. The
@@ -408,7 +448,7 @@ class ProxyObject:
         this won't be a problem.
         """
 
-        ret = self.ProxyMethodClass(self, self._bus.get_connection(),
+        ret = self.ProxyMethodClass(self, self._bus,
                                     self._named_service,
                                     self.__dbus_object_path__, member,
                                     dbus_interface)
@@ -428,3 +468,80 @@ class ProxyObject:
             self._bus, self._named_service, self.__dbus_object_path__, id(self))
     __str__ = __repr__
 
+
+class Interface(object):
+    """An interface into a remote object.
+
+    An Interface can be used to wrap ProxyObjects
+    so that calls can be routed to their correct
+    D-Bus interface.
+    """
+
+    def __init__(self, object, dbus_interface):
+        """Construct a proxy for the given interface on the given object.
+
+        :Parameters:
+            `object` : `dbus.proxies.ProxyObject` or `dbus.Interface`
+                The remote object or another of its interfaces
+            `dbus_interface` : str
+                An interface the `object` implements
+        """
+        if isinstance(object, Interface):
+            self._obj = object.proxy_object
+        else:
+            self._obj = object
+        self._dbus_interface = dbus_interface
+
+    object_path = property (lambda self: self._obj.object_path, None, None,
+                            "The D-Bus object path of the underlying object")
+    __dbus_object_path__ = object_path
+    bus_name = property (lambda self: self._obj.bus_name, None, None,
+                         "The bus name to which the underlying proxy object "
+                         "is bound")
+    requested_bus_name = property (lambda self: self._obj.requested_bus_name,
+                                   None, None,
+                                   "The bus name which was requested when the "
+                                   "underlying object was created")
+    proxy_object = property (lambda self: self._obj, None, None,
+                             """The underlying proxy object""")
+    dbus_interface = property (lambda self: self._dbus_interface, None, None,
+                               """The D-Bus interface represented""")
+
+    def connect_to_signal(self, signal_name, handler_function,
+                          dbus_interface=None, **keywords):
+        """Arrange for a function to be called when the given signal is
+        emitted.
+
+        The parameters and keyword arguments are the same as for
+        `dbus.proxies.ProxyObject.connect_to_signal`, except that if
+        `dbus_interface` is None (the default), the D-Bus interface that
+        was passed to the `Interface` constructor is used.
+        """
+        if not dbus_interface:
+            dbus_interface = self._dbus_interface
+
+        return self._obj.connect_to_signal(signal_name, handler_function,
+                                           dbus_interface, **keywords)
+
+    def __getattr__(self, member):
+        if member.startswith('__') and member.endswith('__'):
+            raise AttributeError(member)
+        else:
+            return self._obj.get_dbus_method(member, self._dbus_interface)
+
+    def get_dbus_method(self, member, dbus_interface=None):
+        """Return a proxy method representing the given D-Bus method.
+
+        This is the same as `dbus.proxies.ProxyObject.get_dbus_method`
+        except that if `dbus_interface` is None (the default),
+        the D-Bus interface that was passed to the `Interface` constructor
+        is used.
+        """
+        if dbus_interface is None:
+            dbus_interface = self._dbus_interface
+        return self._obj.get_dbus_method(member, dbus_interface)
+
+    def __repr__(self):
+        return '<Interface %r implementing %r at %#x>'%(
+        self._obj, self._dbus_interface, id(self))
+    __str__ = __repr__
