@@ -28,11 +28,12 @@ import operator
 import traceback
 
 import _dbus_bindings
-import dbus._dbus as _dbus
+from dbus import SessionBus
 from dbus.exceptions import NameExistsException
 from dbus.exceptions import UnknownMethodException
 from dbus.decorators import method
 from dbus.decorators import signal
+from dbus.proxies import LOCAL_PATH
 
 
 _logger = logging.getLogger('dbus.service')
@@ -89,9 +90,12 @@ class BusName(object):
                 services waiting for the requested name if another service
                 already holds it.
         """
+        _dbus_bindings.validate_bus_name(name, allow_well_known=True,
+                                         allow_unique=False)
+
         # get default bus
         if bus == None:
-            bus = _dbus.Bus()
+            bus = SessionBus()
 
         # see if this name is already defined, return it if so
         # FIXME: accessing internals of Bus
@@ -99,7 +103,7 @@ class BusName(object):
             return bus._bus_names[name]
 
         # otherwise register the name
-	name_flags = (
+        name_flags = (
             (allow_replacement and _dbus_bindings.NAME_FLAG_ALLOW_REPLACEMENT or 0) |
             (replace_existing and _dbus_bindings.NAME_FLAG_REPLACE_EXISTING or 0) |
             (do_not_queue and _dbus_bindings.NAME_FLAG_DO_NOT_QUEUE or 0))
@@ -224,13 +228,12 @@ def _method_lookup(self, method_name, dbus_interface):
             raise UnknownMethodException('%s is not a valid method' % method_name)
 
 
-# FIXME: if signature is '', we used to accept anything, but now insist on
-# zero args. Which was desired? -smcv
 def _method_reply_return(connection, message, method_name, signature, *retval):
     reply = _dbus_bindings.MethodReturnMessage(message)
     try:
         reply.append(signature=signature, *retval)
     except Exception, e:
+        logging.basicConfig()
         if signature is None:
             try:
                 signature = reply.guess_signature(retval) + ' (guessed)'
@@ -396,6 +399,10 @@ class Object(Interface):
         """
         if object_path is None:
             raise TypeError('The object_path argument is required')
+        _dbus_bindings.validate_object_path(object_path)
+        if object_path == LOCAL_PATH:
+            raise DBusException('Objects may not be exported on the reserved '
+                                'path %s' % LOCAL_PATH)
 
         if isinstance(conn, BusName):
             # someone's using the old API; don't gratuitously break them
@@ -409,9 +416,7 @@ class Object(Interface):
 
         self._object_path = object_path
         self._name = bus_name
-        self._bus = conn
-            
-        self._connection = self._bus.get_connection()
+        self._connection = conn
 
         self._connection._register_object_path(object_path, self._message_cb, self._unregister_cb)
 
@@ -443,9 +448,15 @@ class Object(Interface):
                 keywords[return_callback] = lambda *retval: _method_reply_return(connection, message, method_name, signature, *retval)
                 keywords[error_callback] = lambda exception: _method_reply_error(connection, message, exception)
 
-            # include the sender if desired
+            # include the sender etc. if desired
             if parent_method._dbus_sender_keyword:
                 keywords[parent_method._dbus_sender_keyword] = message.get_sender()
+            if parent_method._dbus_path_keyword:
+                keywords[parent_method._dbus_path_keyword] = message.get_path()
+            if parent_method._dbus_destination_keyword:
+                keywords[parent_method._dbus_destination_keyword] = message.get_destination()
+            if parent_method._dbus_message_keyword:
+                keywords[parent_method._dbus_message_keyword] = message
 
             # call method
             retval = candidate_method(self, *args, **keywords)
@@ -480,8 +491,14 @@ class Object(Interface):
 
             # no signature, so just turn the return into a tuple and send it as normal
             else:
-                if retval == None:
+                if retval is None:
                     retval = ()
+                elif (isinstance(retval, tuple)
+                      and not isinstance(retval, _dbus_bindings.Struct)):
+                # If the return is a tuple that is not a Struct, we use it
+                # as-is on the assumption that there are multiple return
+                # values - this is the usual Python idiom. (fd.o #10174)
+                    pass
                 else:
                     retval = (retval,)
 
