@@ -60,11 +60,25 @@ test_types_vals = [1, 12323231, 3.14159265, 99999999.99,
                  #([1,2,3],"c", 1.2, ["a","b","c"], {"a": (1,"v"), "b": (2,"d")})
                  ]
 
+NAME = "org.freedesktop.DBus.TestSuitePythonService"
+IFACE = "org.freedesktop.DBus.TestSuiteInterface"
+OBJECT = "/org/freedesktop/DBus/TestSuitePythonObject"
+
 class TestDBusBindings(unittest.TestCase):
     def setUp(self):
         self.bus = dbus.SessionBus()
-        self.remote_object = self.bus.get_object("org.freedesktop.DBus.TestSuitePythonService", "/org/freedesktop/DBus/TestSuitePythonObject")
-        self.iface = dbus.Interface(self.remote_object, "org.freedesktop.DBus.TestSuiteInterface")
+        self.remote_object = self.bus.get_object(NAME, OBJECT)
+        self.iface = dbus.Interface(self.remote_object, IFACE)
+
+    def testGObject(self):
+        print "Testing ExportedGObject... ",
+        remote_gobject = self.bus.get_object(NAME, OBJECT + '/GObject')
+        iface = dbus.Interface(remote_gobject, IFACE)
+        print "introspection, ",
+        remote_gobject.Introspect(dbus_interface=dbus.INTROSPECTABLE_IFACE)
+        print "method call, ",
+        self.assertEquals(iface.Echo('123'), '123')
+        print "... OK"
 
     def testWeakRefs(self):
         # regression test for Sugar crash caused by smcv getting weak refs
@@ -112,6 +126,17 @@ class TestDBusBindings(unittest.TestCase):
             self.assertEquals(send_val, recv_val)
             self.assertEquals(recv_val.variant_level, 1)
 
+    def testMethodExtraInfoKeywords(self):
+        print "Testing MethodExtraInfoKeywords..."
+        sender, path, destination, message_cls = self.iface.MethodExtraInfoKeywords()
+        self.assert_(sender.startswith(':'))
+        self.assertEquals(path, '/org/freedesktop/DBus/TestSuitePythonObject')
+        # we're using the "early binding" form of get_object (without
+        # follow_name_owner_changes), so the destination we actually sent it
+        # to will be the unique name
+        self.assert_(destination.startswith(':'))
+        self.assertEquals(message_cls, 'dbus.lowlevel.MethodCallMessage')
+
     def testUtf8StringsSync(self):
         send_val = u'foo'
         recv_val = self.iface.Echo(send_val, utf8_strings=True)
@@ -123,12 +148,12 @@ class TestDBusBindings(unittest.TestCase):
 
     def testBenchmarkIntrospect(self):
         print "\n********* Benchmark Introspect ************"
-	a = time.time()
-	print a
+        a = time.time()
+        print a
         print self.iface.GetComplexArray()
-	b = time.time()
-	print b
-	print "Delta: %f" % (b - a)
+        b = time.time()
+        print b
+        print "Delta: %f" % (b - a)
         self.assert_(True)
 
     def testAsyncCalls(self):
@@ -296,6 +321,22 @@ class TestDBusBindings(unittest.TestCase):
         myself = self.iface.WhoAmI()
         print "I am", myself
 
+    def testBusGetNameOwner(self):
+        ret = self.bus.get_name_owner(NAME)
+        self.assert_(ret.startswith(':'), ret)
+
+    def testBusListNames(self):
+        ret = self.bus.list_names()
+        self.assert_(NAME in ret, ret)
+
+    def testBusListActivatableNames(self):
+        ret = self.bus.list_activatable_names()
+        self.assert_(NAME in ret, ret)
+
+    def testBusNameHasOwner(self):
+        self.assert_(self.bus.name_has_owner(NAME))
+        self.assert_(not self.bus.name_has_owner('badger.mushroom.snake'))
+
     def testBusNameCreation(self):
         print '\n******** Testing BusName creation ********'
         test = [('org.freedesktop.DBus.Python.TestName', True),
@@ -336,15 +377,21 @@ class TestDBusBindings(unittest.TestCase):
         ret = bus.name_has_owner('org.freedesktop.DBus.Python.TestName')
         self.assert_(not ret, 'deleting reference failed to release BusName org.freedesktop.DBus.Python.TestName')
 
+    def testMultipleReturnWithoutSignature(self):
+        # https://bugs.freedesktop.org/show_bug.cgi?id=10174
+        ret = self.iface.MultipleReturnWithoutSignature()
+        self.assert_(not isinstance(ret, dbus.Struct), repr(ret))
+        self.assertEquals(ret, ('abc', 123))
+
 """ Remove this for now
 class TestDBusPythonToGLibBindings(unittest.TestCase):
     def setUp(self):
         self.bus = dbus.SessionBus()
         self.remote_object = self.bus.get_object("org.freedesktop.DBus.TestSuiteGLibService", "/org/freedesktop/DBus/Tests/MyTestObject")
         self.iface = dbus.Interface(self.remote_object, "org.freedesktop.DBus.Tests.MyObject")
-			    
+
     def testIntrospection(self):
-	#test introspection
+        #test introspection
         print "\n********* Introspection Test ************"
         print self.remote_object.Introspect(dbus_interface="org.freedesktop.DBus.Introspectable")
         print "Introspection test passed"
@@ -355,7 +402,7 @@ class TestDBusPythonToGLibBindings(unittest.TestCase):
         result =  self.iface.ManyArgs(1000, 'Hello GLib', 2)
         print result
         self.assert_(result == [2002.0, 'HELLO GLIB'])
-	
+
         arg0 = {"Dude": 1, "john": "palmieri", "python": 2.4}
         result = self.iface.ManyStringify(arg0)
         print result
